@@ -6,13 +6,17 @@ use crate::{types, Buffer};
 pub struct BufferReader<'a, 'b> {
     raw_buffer: &'a Buffer,
     data_types: &'b [u8],
+
+    field_pos_index: Vec<usize>,
 }
 
 impl<'a, 'b> BufferReader<'a, 'b> {
     pub(crate) fn new(raw_buffer: &'a Buffer, data_types: &'b [u8]) -> Self {
+        let field_pos_index = build_position_index(raw_buffer, data_types);
         BufferReader {
             data_types,
             raw_buffer,
+            field_pos_index,
         }
     }
 
@@ -23,7 +27,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
         field_len: usize,
         data_type: u8,
     ) -> Result<(), std::io::Error> {
-        if self.raw_buffer.field_pos_index[index] + field_len > self.raw_buffer.buf_len {
+        if self.field_pos_index[index] + field_len > self.raw_buffer.buf_len {
             return Err(std::io::Error::from(ErrorKind::UnexpectedEof));
         }
 
@@ -37,7 +41,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_bool(&self, index: usize) -> Result<bool, std::io::Error> {
         self.index_out_of_bounds_check(index, 1, types::BOOL)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -51,7 +55,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_i8(&self, index: usize) -> Result<i8, std::io::Error> {
         self.index_out_of_bounds_check(index, 1, types::I8)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -65,7 +69,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_u8(&self, index: usize) -> Result<u8, std::io::Error> {
         self.index_out_of_bounds_check(index, 1, types::U8)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -79,7 +83,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_i16(&self, index: usize) -> Result<i16, std::io::Error> {
         self.index_out_of_bounds_check(index, 2, types::I16)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -93,7 +97,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_u16(&self, index: usize) -> Result<u16, std::io::Error> {
         self.index_out_of_bounds_check(index, 2, types::U16)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -107,7 +111,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_i32(&self, index: usize) -> Result<i32, std::io::Error> {
         self.index_out_of_bounds_check(index, 4, types::I32)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -121,7 +125,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_u32(&self, index: usize) -> Result<u32, std::io::Error> {
         self.index_out_of_bounds_check(index, 4, types::U32)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -135,7 +139,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_i64(&self, index: usize) -> Result<i64, std::io::Error> {
         self.index_out_of_bounds_check(index, 8, types::I64)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -149,7 +153,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_u64(&self, index: usize) -> Result<u64, std::io::Error> {
         self.index_out_of_bounds_check(index, 8, types::U64)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -163,7 +167,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_f32(&self, index: usize) -> Result<f32, std::io::Error> {
         self.index_out_of_bounds_check(index, 4, types::F32)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -177,7 +181,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     pub fn get_f64(&self, index: usize) -> Result<f64, std::io::Error> {
         self.index_out_of_bounds_check(index, 8, types::F64)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -201,7 +205,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
     }
 
     fn get_bytes(&self, index: usize, data_type_id: u8) -> Result<&'a [u8], std::io::Error> {
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let (v, len_length) = read_lenenc_int(&self.raw_buffer.buf, start)?;
 
         let len = v as usize;
@@ -219,7 +223,7 @@ impl<'a, 'b> BufferReader<'a, 'b> {
             self.get_bytes(index, data_type_id)
         } else {
             let len = types::len(data_type_id) as usize;
-            let start = self.raw_buffer.field_pos_index[index];
+            let start = self.field_pos_index[index];
 
             let s = self.raw_buffer.buf.get(start..start + len).unwrap();
 
@@ -233,13 +237,17 @@ impl<'a, 'b> BufferReader<'a, 'b> {
 pub struct BufferMutReader<'a, 'b> {
     raw_buffer: &'a mut Buffer,
     data_types: &'b [u8],
+
+    field_pos_index: Vec<usize>,
 }
 
 impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub(crate) fn new(raw_buffer: &'a mut Buffer, data_types: &'b [u8]) -> Self {
+        let field_pos_index = build_position_index(raw_buffer, data_types);
         BufferMutReader {
             data_types,
             raw_buffer,
+            field_pos_index,
         }
     }
 
@@ -250,7 +258,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
         field_len: usize,
         data_type: u8,
     ) -> Result<(), std::io::Error> {
-        if self.raw_buffer.field_pos_index[index] + field_len > self.raw_buffer.buf_len {
+        if self.field_pos_index[index] + field_len > self.raw_buffer.buf_len {
             return Err(std::io::Error::from(ErrorKind::UnexpectedEof));
         }
 
@@ -264,7 +272,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_bool(&mut self, index: usize) -> Result<bool, std::io::Error> {
         self.index_out_of_bounds_check(index, 1, types::BOOL)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -278,7 +286,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_i8(&mut self, index: usize) -> Result<i8, std::io::Error> {
         self.index_out_of_bounds_check(index, 1, types::I8)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -292,7 +300,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_u8(&mut self, index: usize) -> Result<u8, std::io::Error> {
         self.index_out_of_bounds_check(index, 1, types::U8)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -306,7 +314,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_i16(&mut self, index: usize) -> Result<i16, std::io::Error> {
         self.index_out_of_bounds_check(index, 2, types::I16)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -320,7 +328,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_u16(&mut self, index: usize) -> Result<u16, std::io::Error> {
         self.index_out_of_bounds_check(index, 2, types::U16)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -334,7 +342,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_i32(&mut self, index: usize) -> Result<i32, std::io::Error> {
         self.index_out_of_bounds_check(index, 4, types::I32)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -348,7 +356,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_u32(&mut self, index: usize) -> Result<u32, std::io::Error> {
         self.index_out_of_bounds_check(index, 4, types::U32)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -362,7 +370,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_i64(&mut self, index: usize) -> Result<i64, std::io::Error> {
         self.index_out_of_bounds_check(index, 8, types::I64)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -376,7 +384,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_u64(&mut self, index: usize) -> Result<u64, std::io::Error> {
         self.index_out_of_bounds_check(index, 8, types::U64)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -390,7 +398,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_f32(&mut self, index: usize) -> Result<f32, std::io::Error> {
         self.index_out_of_bounds_check(index, 4, types::F32)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -404,7 +412,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     pub fn get_f64(&mut self, index: usize) -> Result<f64, std::io::Error> {
         self.index_out_of_bounds_check(index, 8, types::F64)?;
 
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let s = self
             .raw_buffer
             .buf
@@ -432,7 +440,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
     }
 
     fn get_bytes(&mut self, index: usize, data_type_id: u8) -> Result<&[u8], std::io::Error> {
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let (v, len_length) = read_lenenc_int(&self.raw_buffer.buf, start)?;
 
         let len = v as usize;
@@ -448,7 +456,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
         index: usize,
         data_type_id: u8,
     ) -> Result<&mut [u8], std::io::Error> {
-        let start = self.raw_buffer.field_pos_index[index];
+        let start = self.field_pos_index[index];
         let (v, len_length) = read_lenenc_int(&self.raw_buffer.buf, start)?;
 
         let len = v as usize;
@@ -466,7 +474,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
             self.get_bytes(index, data_type_id)
         } else {
             let len = types::len(data_type_id) as usize;
-            let start = self.raw_buffer.field_pos_index[index];
+            let start = self.field_pos_index[index];
 
             let s = self.raw_buffer.buf.get(start..start + len).unwrap();
 
@@ -480,7 +488,7 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
             self.get_bytes_mut(index, data_type_id)
         } else {
             let len = types::len(data_type_id) as usize;
-            let start = self.raw_buffer.field_pos_index[index];
+            let start = self.field_pos_index[index];
 
             let s = self.raw_buffer.buf.get_mut(start..start + len).unwrap();
 
@@ -488,3 +496,31 @@ impl<'a, 'b> BufferMutReader<'a, 'b> {
         }
     }
 }
+
+fn build_position_index(raw_buffer: &Buffer, data_types: &[u8]) -> Vec<usize> {
+    let mut field_pos_index = Vec::with_capacity(data_types.len());
+    let mut field_start_pos = 0;
+    for index in 0..data_types.len() {
+        if field_start_pos > raw_buffer.buf_len {
+            panic!("read error");
+        }
+
+        field_pos_index.push(field_start_pos);
+        let data_type = data_types[index];
+        if data_type >= types::BINARY {
+            let (v, len_length) = read_lenenc_int(&raw_buffer.buf, field_start_pos).unwrap();
+            let len = v as usize;
+
+            field_start_pos += (len + len_length) as usize;
+        } else {
+            let len = types::len(data_type);
+            field_start_pos += len as usize;
+        }
+    }
+
+    if field_start_pos > raw_buffer.buf_len {
+        panic!("read error");
+    }
+
+    field_pos_index
+}
